function [ xlut, ylut, zlut ] = LM_XYZLUT( fits )
% This file is part of LM.
% 
% LM is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% Foobar is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with LM.  If not, see <http://www.gnu.org/licenses/>.
 

    %Z
    poly_order=3;
    rawfits=fits;

    %normalise
    meanZ=mean(fits(:,8));
    fits(:,8)=fits(:,8)-meanZ;
    meanZ=0;
    Z=fits(:,8);
    
    %calauclate shape ratios
    RAT=fits(:,5)./fits(:,3);
    
    %cubic fit
    p=polyfit(RAT,Z,poly_order);
    
    %geneate LUT
    RAT_min=min(RAT);
    RAT_step=0.01;
    RAT_max=max(RAT);    
    RAT_lookup=[RAT_min:RAT_step:RAT_max];    
    Z_lookup = polyval(p,RAT_lookup);
            
    %make z=0 when ratio=1
    [~,i] = min(abs(RAT_lookup-1));
    Z_norm=Z_lookup-Z_lookup(i);
    
    %build lookup table
    zlut=vertcat(RAT_lookup,Z_norm);
    zlut=zlut.';  

    %XY      
    X_offsets=fits(:,2)-fits(i,2);
    [pX,SX]=polyfit(RAT,X_offsets,1);
    X_lookup = polyval(pX,RAT_lookup);
    xlut=vertcat(RAT_lookup,X_lookup);
    xlut=xlut.';
    
    Y_offsets=fits(:,4)-fits(i,4);
    [pY,SY]=polyfit(RAT,Y_offsets,1);
    Y_lookup = polyval(pY,RAT_lookup);
    ylut=vertcat(RAT_lookup,Y_lookup);
    ylut=ylut.';
    
    %check calibration
    rawfits=LM_lookupXYZ(rawfits,xlut,ylut,zlut);
    
    %plot output    
    subplot(2,2,1);
    hold on
        scatter(RAT,Z,'b');
        scatter(RAT_lookup,Z_lookup,'r');
        scatter(RAT_lookup,Z_norm,'g');
        legend('raw','fitted','normalised so ratio=1 : z=0');
        title('lookup table')    
    hold off
    
    subplot(2,2,2);    
    %scatter(fits(:,5)./fits(:,3),fits(:,1))
    %title('intensity by asymmetry')
    plot(fits(:,8))
    title('pizeo position by frame')
    
    
    subplot(2,2,3);    
    scatter(rawfits(:,8),rawfits(:,9))
    title('z positions determined using calibration (reverse loopkup)')
    
    
    subplot(2,2,4); 
    %{
    hold on
    scatter(RAT_lookup,X_lookup*94,'k');
    scatter(RAT,X_offsets*94,'k');
    scatter(RAT_lookup,Y_lookup*94,'b');
    scatter(RAT,Y_offsets*94,'b');
    hold off
    %}
    hold on
    plot(rawfits(:,2)*94,'b');
    plot(rawfits(:,22)*94,'k');
    plot(rawfits(:,4)*94,'b');
    plot(rawfits(:,24)*94,'k');
    hold off
    
    %{
    Z_raw_norm=fits(:,8)-fits(i,8);
    scatter(X_norm*94,Y_norm*94,20,Z_raw_norm);
    daspect([1 1 1]);
    title('x vs y position, coloured by z position');
    xlabel('x position (nm)');
    ylabel('y position (nm)');
    %}
    
    std(rawfits(:,2))
    std(rawfits(:,22))
    std(rawfits(:,4))
    std(rawfits(:,24))
    
end

