function [ zlut ] = LM_ZLUT( fits )
% This file is part of LM.
% 
% LM is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% Foobar is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with LM.  If not, see <http://www.gnu.org/licenses/>.
 
    poly_order=3;

    rawfits=fits;

    %normalise
    meanZ=mean(fits(:,8));
    fits(:,8)=fits(:,8)-meanZ;
    meanZ=0;
    Z=fits(:,8);
    
    %calauclate shape ratios
    RAT=fits(:,5)./fits(:,3);
    
    %cubic fit
    p=polyfit(RAT,Z,poly_order);
    
    %geneate LUT
    RAT_min=min(RAT);
    RAT_step=0.01;
    RAT_max=max(RAT);    
    RAT_lookup=[RAT_min:RAT_step:RAT_max];    
    Z_lookup = polyval(p,RAT_lookup);
    
    %{
    hold on
    scatter(RAT,Z);
    scatter(RAT_lookup,Z_lookup,'r');
    hold off
    %}
    
    %make z=0 when ratio=1
    [~,i] = min(abs(RAT_lookup-1));
    Z_norm=Z_lookup-Z_lookup(i);
    
    %build lookup table
    zlut=vertcat(RAT_lookup,Z_norm);
    zlut=zlut.';  
    
    %check calibration
    rawfits=LM_lookupZ(rawfits,zlut);
    
    
    %plot output    
    subplot(2,2,1);
    hold on
        scatter(RAT,Z,'b');
        scatter(RAT_lookup,Z_lookup,'r');
        scatter(RAT_lookup,Z_norm,'g');
        legend('raw','fitted','normalised so ratio=1 : z=0');
        title('lookup table')    
    hold off
    
    subplot(2,2,2);    
    %scatter(fits(:,5)./fits(:,3),fits(:,1))
    %title('intensity by asymmetry')
    plot(fits(:,8))
    title('pizeo position by frame')
    
    
    subplot(2,2,3);    
    scatter(rawfits(:,8),rawfits(:,9))
    title('z positions determined using calibration (reverse loopkup)')
    
    subplot(2,2,4);    
    X_norm=fits(:,2)-fits(i,2);
    Y_norm=fits(:,4)-fits(i,4);
    Z_raw_norm=fits(:,8)-fits(i,8);
    scatter(X_norm*94,Y_norm*94,20,Z_raw_norm);
    daspect([1 1 1]);
    title('x vs y position, coloured by z position');
    xlabel('x position (nm)');
    ylabel('y position (nm)');
    
end

