function [ offsets ] = LM_getdrift( fits )
% This file is part of LM.
% 
% LM is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% Foobar is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with LM.  If not, see <http://www.gnu.org/licenses/>.

    %count number of events per emitter
    tab=tabulate(fits(:,10));
    %find emitter with most events
    tab=flipdim(sortrows(tab,2),1);
    mainemitter=tab(1,1)
    %return events associated with that emitter
    driftevents=fits(fits(:,10)==mainemitter,:);
    %make sure there's a maximum of one fit per frame
    [~, unirows, ~]=unique(driftevents(:,7));
    driftevents=driftevents(unirows,:);
    %interpolate so there is one fit per frame
    deinterp=interp1(driftevents(:,7),driftevents,driftevents(1,7):1:driftevents(size(driftevents,1),7));
    
    'selected reference point co-ordinates'
    'x'
    deinterp(1,2)
    'y'
    deinterp(1,4)
    'z'
    deinterp(1,9)
    
    %x offset
    deinterp(:,2)=deinterp(:,2)-deinterp(1,2);
    %y offset
    deinterp(:,4)=deinterp(:,4)-deinterp(1,4);
    %z offset
    deinterp(:,9)=deinterp(:,9)-deinterp(1,9);
    
    offsets=deinterp;
    
    %Savitzky-Golay filtering smooths the offset trace, effectively
    %averaging and improving resolution
    offsets(:,[2,4,9])=sgolayfilt(offsets(:,[2,4,9]),3,501);
       
        
end

