function myplot=LM_simplerender(fits, columns, path, renderset)
% This file is part of LM.
% 
% LM is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% Foobar is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with LM.  If not, see <http://www.gnu.org/licenses/>.

%columns should be a 1D matrix of length three, specifying X, Y, and Z data
%columns for display, in that order

X1=fits(:,columns(1));
Y1=fits(:,columns(2));
Z1=fits(:,columns(3));

if size(columns,2)>3
    Z2=fits(:,columns(4));
else
    Z2=Z1;
end

if nargin<4
    renderset=LM_MakeRenderset(X1, Y1, Z1)
end

if size(fits,2)>20
    channels=max(fits(:,21));
    renderset.zscalemin=1;
    renderset.zscalemax=channels;
else
    channels=1;
end

%filter by Z
X1=X1(Z1>=renderset.zmin & Z1<=renderset.zmax);
Y1=Y1(Z1>=renderset.zmin & Z1<=renderset.zmax);
Z1=Z1(Z1>=renderset.zmin & Z1<=renderset.zmax);

% Create figure
figure1 = figure('Colormap',renderset.map,...
    'Color',renderset.bgcolor);
    %[0 0 0;0 0 0.142857149243355;0 0 0.28571429848671;0 0 0.428571432828903;0 0 0.571428596973419;0 0 0.714285731315613;0 0 0.857142865657806;0 0 1;0 0.0625 1;0 0.125 1;0 0.1875 1;0 0.25 1;0 0.3125 1;0 0.375 1;0 0.4375 1;0 0.5 1;0 0.5625 1;0 0.625 1;0 0.6875 1;0 0.75 1;0 0.8125 1;0 0.875 1;0 0.9375 1;0 1 1;0.0625 1 0.9375;0.125 1 0.875;0.1875 1 0.8125;0.25 1 0.75;0.3125 1 0.6875;0.375 1 0.625;0.4375 1 0.5625;0.5 1 0.5;0.5625 1 0.4375;0.625 1 0.375;0.6875 1 0.3125;0.75 1 0.25;0.8125 1 0.1875;0.875 1 0.125;0.9375 1 0.0625;1 1 0;1 0.9375 0;1 0.875 0;1 0.8125 0;1 0.75 0;1 0.6875 0;1 0.625 0;1 0.5625 0;1 0.5 0;1 0.4375 0;1 0.375 0;1 0.3125 0;1 0.25 0;1 0.1875 0;1 0.125 0;1 0.0625 0;1 0 0;0.875 0 0;0.75 0 0;0.625 0 0;0.5 0 0;0.375 0 0;0.25 0 0;0.125 0 0;0 0 0],...

% Create axes
axes1 = axes('Parent',figure1,...
    'ZColor',renderset.axescolor,...
    'YColor',renderset.axescolor,...
    'XColor',renderset.axescolor,...
    'DataAspectRatio',[1 1 1],...
    'Color',renderset.bgcolor);
% Uncomment the following line to preserve the X-limits of the axes
% xlim(axes1,[0 62]);
% Uncomment the following line to preserve the Y-limits of the axes
% ylim(axes1,[0 46]);
box(axes1,'on');
hold(axes1,'all');

% Create scatter
if renderset.threed==1
    ms(1:size(X1,1))=renderset.pointsize;
    myplot=scatter3(X1*renderset.pixelsize,Y1*renderset.pixelsize,Z1,ms,Z2,'fill'); %,'MarkerEdgeColor','black');
    camproj('perspective');
    axis(gca,'vis3d');
else
    myplot=scatter(X1*renderset.pixelsize,Y1*renderset.pixelsize,renderset.pointsize,Z1,'filled'); %,'MarkerEdgeColor','black');
    %myplot=scatter(X1*renderset.pixelsize,Y1*renderset.pixelsize,1,[0 0 0],'.'); %,'MarkerEdgeColor','black');
end

%set axis limits
axis([renderset.xmin*renderset.pixelsize renderset.xmax*renderset.pixelsize renderset.ymin*renderset.pixelsize renderset.ymax*renderset.pixelsize])
set(gca,'YDir','reverse')

%set 1:1:1 aspect ratio
set(gca,'DataAspectRatio',[1 1 1])

%set background to black even when exporting
set(gcf,'InvertHardCopy','off');

%set colormap scale
if renderset.zscalemin==renderset.zscalemax
else
    set(gca, 'CLim', [renderset.zscalemin, renderset.zscalemax]);
end

%show colourbar
cb=colorbar;
set(cb,'Ycolor',renderset.axescolor);

%label data points
if renderset.label==1
    dx = renderset.pixelsize/10; dy = renderset.pixelsize/10; % displacement so the text does not overlay the data points
    text((X1+dx)*renderset.pixelsize, (Y1+dy)*renderset.pixelsize, num2str(Z1),'color',[0.749019622802734 0.749019622802734 0])
end

%set colormap
if channels==1
    colormap(renderset.map)
    %colormap=renderset.map;
else
    colormap(renderset.map2)
    %colormap=renderset.map2;
end

colormapeditor

% enable data cursor mode
dcm_obj = datacursormode(figure1);
set(dcm_obj,'UpdateFcn',@LM_DCUpdateFcn)
set(dcm_obj, 'enable', 'on')

%should make pos available to main and nested functions
pos=[];

    function txt = LM_DCUpdateFcn(~, event_obj)
            % Customizes text of data tips

            % read out data point
            pos = get(event_obj,'Position');                                   
            
            % no data shown on figure
            txt = {'Selected event.'};

    end

    ht = uitoolbar(figure1);
    % Use a MATLAB icon for the tool
    [X map] = imread(fullfile(...
        matlabroot,'toolbox','matlab','icons','matlabicon.gif'));
    % Convert indexed image and colormap to truecolor
    icon = ind2rgb(X,map);
    % Create a uipushtool in the toolbar
    hpt = uipushtool(ht,'CData',icon,...
                     'TooltipString','Toolbar push button',...
                     'ClickedCallback', @LM_showsource);
       
    function LM_showsource(~, event_obj)
       myfit=fits(fits(:,columns(1))==pos(1) & fits(:,columns(2))==pos(2),:)
       figure
       LM_checkframe( myfit(7), fits, path )
    end

    myplot=get(myplot,'children');

end

